package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLAttribMask implements GLEnum {
	CURRENT_BIT(0x1),
	POINT_BIT(0x2),
	LINE_BIT(0x4),
	POLYGON_BIT(0x8),
	POLYGON_STIPPLE_BIT(0x10),
	PIXEL_MODE_BIT(0x20),
	LIGHTING_BIT(0x40),
	FOG_BIT(0x80),
	DEPTH_BUFFER_BIT(0x100),
	ACCUM_BUFFER_BIT(0x200),
	STENCIL_BUFFER_BIT(0x400),
	VIEWPORT_BIT(0x800),
	TRANSFORM_BIT(0x1000),
	ENABLE_BIT(0x2000),
	COLOR_BUFFER_BIT(0x4000),
	HINT_BIT(0x8000),
	EVAL_BIT(0x10000),
	LIST_BIT(0x20000),
	TEXTURE_BIT(0x40000),
	SCISSOR_BIT(0x80000),
	MULTISAMPLE_BIT(0x20000000),
	ALL_ATTRIB_BITS(0xffffffff);
	
	private final int glEnum;

	GLAttribMask(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLAttribMask> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLAttribMask fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1 -> CURRENT_BIT;
			case 0x2 -> POINT_BIT;
			case 0x4 -> LINE_BIT;
			case 0x8 -> POLYGON_BIT;
			case 0x10 -> POLYGON_STIPPLE_BIT;
			case 0x20 -> PIXEL_MODE_BIT;
			case 0x40 -> LIGHTING_BIT;
			case 0x80 -> FOG_BIT;
			case 0x100 -> DEPTH_BUFFER_BIT;
			case 0x200 -> ACCUM_BUFFER_BIT;
			case 0x400 -> STENCIL_BUFFER_BIT;
			case 0x800 -> VIEWPORT_BIT;
			case 0x1000 -> TRANSFORM_BIT;
			case 0x2000 -> ENABLE_BIT;
			case 0x4000 -> COLOR_BUFFER_BIT;
			case 0x8000 -> HINT_BIT;
			case 0x10000 -> EVAL_BIT;
			case 0x20000 -> LIST_BIT;
			case 0x40000 -> TEXTURE_BIT;
			case 0x80000 -> SCISSOR_BIT;
			case 0x20000000 -> MULTISAMPLE_BIT;
			case 0xffffffff -> ALL_ATTRIB_BITS;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLAttribMask");
		};
	}
}