package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLBufferStorageMask implements GLEnum {
	MAP_READ_BIT(0x1),
	MAP_WRITE_BIT(0x2),
	MAP_PERSISTENT_BIT(0x40),
	MAP_COHERENT_BIT(0x80),
	DYNAMIC_STORAGE_BIT(0x100),
	CLIENT_STORAGE_BIT(0x200),
	SPARSE_STORAGE_BIT_ARB(0x400),
	LGPU_SEPARATE_STORAGE_BIT_NVX(0x800),
	EXTERNAL_STORAGE_BIT_NVX(0x2000);
	
	private final int glEnum;

	GLBufferStorageMask(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLBufferStorageMask> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLBufferStorageMask fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1 -> MAP_READ_BIT;
			case 0x2 -> MAP_WRITE_BIT;
			case 0x40 -> MAP_PERSISTENT_BIT;
			case 0x80 -> MAP_COHERENT_BIT;
			case 0x100 -> DYNAMIC_STORAGE_BIT;
			case 0x200 -> CLIENT_STORAGE_BIT;
			case 0x400 -> SPARSE_STORAGE_BIT_ARB;
			case 0x800 -> LGPU_SEPARATE_STORAGE_BIT_NVX;
			case 0x2000 -> EXTERNAL_STORAGE_BIT_NVX;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLBufferStorageMask");
		};
	}
}