package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLBufferTarget implements GLEnum {
	PARAMETER_BUFFER(0x80ee),
	ARRAY_BUFFER(0x8892),
	ELEMENT_ARRAY_BUFFER(0x8893),
	PIXEL_PACK_BUFFER(0x88eb),
	PIXEL_UNPACK_BUFFER(0x88ec),
	UNIFORM_BUFFER(0x8a11),
	TEXTURE_BUFFER(0x8c2a),
	TRANSFORM_FEEDBACK_BUFFER(0x8c8e),
	COPY_READ_BUFFER(0x8f36),
	COPY_WRITE_BUFFER(0x8f37),
	DRAW_INDIRECT_BUFFER(0x8f3f),
	SHADER_STORAGE_BUFFER(0x90d2),
	DISPATCH_INDIRECT_BUFFER(0x90ee),
	QUERY_BUFFER(0x9192),
	ATOMIC_COUNTER_BUFFER(0x92c0);
	
	private final int glEnum;

	GLBufferTarget(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLBufferTarget> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLBufferTarget fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x80ee -> PARAMETER_BUFFER;
			case 0x8892 -> ARRAY_BUFFER;
			case 0x8893 -> ELEMENT_ARRAY_BUFFER;
			case 0x88eb -> PIXEL_PACK_BUFFER;
			case 0x88ec -> PIXEL_UNPACK_BUFFER;
			case 0x8a11 -> UNIFORM_BUFFER;
			case 0x8c2a -> TEXTURE_BUFFER;
			case 0x8c8e -> TRANSFORM_FEEDBACK_BUFFER;
			case 0x8f36 -> COPY_READ_BUFFER;
			case 0x8f37 -> COPY_WRITE_BUFFER;
			case 0x8f3f -> DRAW_INDIRECT_BUFFER;
			case 0x90d2 -> SHADER_STORAGE_BUFFER;
			case 0x90ee -> DISPATCH_INDIRECT_BUFFER;
			case 0x9192 -> QUERY_BUFFER;
			case 0x92c0 -> ATOMIC_COUNTER_BUFFER;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLBufferTarget");
		};
	}

	/**
	 * If there is a {@link GLGetPName} value that corresponds to this buffer target's
	 * binding, it is returned. Otherwise, {@code null} is returned.
	 */
	public GLGetPName pName() {
		return switch (this) {
			case PARAMETER_BUFFER -> GLGetPName.PARAMETER_BUFFER_BINDING;
			case ARRAY_BUFFER -> GLGetPName.ARRAY_BUFFER_BINDING;
			case ELEMENT_ARRAY_BUFFER -> GLGetPName.ELEMENT_ARRAY_BUFFER_BINDING;
			case PIXEL_PACK_BUFFER -> GLGetPName.PIXEL_PACK_BUFFER_BINDING;
			case PIXEL_UNPACK_BUFFER -> GLGetPName.PIXEL_UNPACK_BUFFER_BINDING;
			case UNIFORM_BUFFER -> GLGetPName.UNIFORM_BUFFER_BINDING;
			case TEXTURE_BUFFER -> GLGetPName.TEXTURE_BUFFER_BINDING;
			case TRANSFORM_FEEDBACK_BUFFER -> GLGetPName.TRANSFORM_FEEDBACK_BUFFER_BINDING;
			case COPY_READ_BUFFER -> GLGetPName.COPY_READ_BUFFER_BINDING;
			case COPY_WRITE_BUFFER -> GLGetPName.COPY_WRITE_BUFFER_BINDING;
			case DRAW_INDIRECT_BUFFER -> GLGetPName.DRAW_INDIRECT_BUFFER_BINDING;
			case SHADER_STORAGE_BUFFER -> GLGetPName.SHADER_STORAGE_BUFFER_BINDING;
			case DISPATCH_INDIRECT_BUFFER -> GLGetPName.DISPATCH_INDIRECT_BUFFER_BINDING;
			case QUERY_BUFFER -> GLGetPName.QUERY_BUFFER_BINDING;
			case ATOMIC_COUNTER_BUFFER -> null;
		};
	}
}