package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLDebugType implements GLEnum {
	DONT_CARE(0x1100),
	DEBUG_TYPE_ERROR(0x824c),
	DEBUG_TYPE_DEPRECATED_BEHAVIOR(0x824d),
	DEBUG_TYPE_UNDEFINED_BEHAVIOR(0x824e),
	DEBUG_TYPE_PORTABILITY(0x824f),
	DEBUG_TYPE_PERFORMANCE(0x8250),
	DEBUG_TYPE_OTHER(0x8251),
	DEBUG_TYPE_MARKER(0x8268),
	DEBUG_TYPE_PUSH_GROUP(0x8269),
	DEBUG_TYPE_POP_GROUP(0x826a);
	
	private final int glEnum;

	GLDebugType(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLDebugType> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLDebugType fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1100 -> DONT_CARE;
			case 0x824c -> DEBUG_TYPE_ERROR;
			case 0x824d -> DEBUG_TYPE_DEPRECATED_BEHAVIOR;
			case 0x824e -> DEBUG_TYPE_UNDEFINED_BEHAVIOR;
			case 0x824f -> DEBUG_TYPE_PORTABILITY;
			case 0x8250 -> DEBUG_TYPE_PERFORMANCE;
			case 0x8251 -> DEBUG_TYPE_OTHER;
			case 0x8268 -> DEBUG_TYPE_MARKER;
			case 0x8269 -> DEBUG_TYPE_PUSH_GROUP;
			case 0x826a -> DEBUG_TYPE_POP_GROUP;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLDebugType");
		};
	}
}