package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLErrorCode implements GLEnum {
	NO_ERROR(0x0),
	INVALID_ENUM(0x500),
	INVALID_VALUE(0x501),
	INVALID_OPERATION(0x502),
	STACK_OVERFLOW(0x503),
	STACK_UNDERFLOW(0x504),
	OUT_OF_MEMORY(0x505),
	INVALID_FRAMEBUFFER_OPERATION(0x506),
	TABLE_TOO_LARGE_EXT(0x8031),
	TEXTURE_TOO_LARGE_EXT(0x8065);
	
	private final int glEnum;

	GLErrorCode(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLErrorCode> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLErrorCode fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x0 -> NO_ERROR;
			case 0x500 -> INVALID_ENUM;
			case 0x501 -> INVALID_VALUE;
			case 0x502 -> INVALID_OPERATION;
			case 0x503 -> STACK_OVERFLOW;
			case 0x504 -> STACK_UNDERFLOW;
			case 0x505 -> OUT_OF_MEMORY;
			case 0x506 -> INVALID_FRAMEBUFFER_OPERATION;
			case 0x8031 -> TABLE_TOO_LARGE_EXT;
			case 0x8065 -> TEXTURE_TOO_LARGE_EXT;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLErrorCode");
		};
	}
}