package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLHintTarget implements GLEnum {
	PERSPECTIVE_CORRECTION_HINT(0xc50),
	POINT_SMOOTH_HINT(0xc51),
	LINE_SMOOTH_HINT(0xc52),
	POLYGON_SMOOTH_HINT(0xc53),
	FOG_HINT(0xc54),
	PACK_CMYK_HINT_EXT(0x800e),
	UNPACK_CMYK_HINT_EXT(0x800f),
	PHONG_HINT_WIN(0x80eb),
	CLIP_VOLUME_CLIPPING_HINT_EXT(0x80f0),
	TEXTURE_MULTI_BUFFER_HINT_SGIX(0x812e),
	GENERATE_MIPMAP_HINT(0x8192),
	PROGRAM_BINARY_RETRIEVABLE_HINT(0x8257),
	CONVOLUTION_HINT_SGIX(0x8316),
	SCALEBIAS_HINT_SGIX(0x8322),
	LINE_QUALITY_HINT_SGIX(0x835b),
	VERTEX_PRECLIP_SGIX(0x83ee),
	VERTEX_PRECLIP_HINT_SGIX(0x83ef),
	TEXTURE_COMPRESSION_HINT(0x84ef),
	VERTEX_ARRAY_STORAGE_HINT_APPLE(0x851f),
	MULTISAMPLE_FILTER_HINT_NV(0x8534),
	TRANSFORM_HINT_APPLE(0x85b1),
	TEXTURE_STORAGE_HINT_APPLE(0x85bc),
	FRAGMENT_SHADER_DERIVATIVE_HINT(0x8b8b),
	BINNING_CONTROL_HINT_QCOM(0x8fb0),
	PREFER_DOUBLEBUFFER_HINT_PGI(0x1a1f8),
	CONSERVE_MEMORY_HINT_PGI(0x1a1fd),
	RECLAIM_MEMORY_HINT_PGI(0x1a1fe),
	NATIVE_GRAPHICS_BEGIN_HINT_PGI(0x1a203),
	NATIVE_GRAPHICS_END_HINT_PGI(0x1a204),
	ALWAYS_FAST_HINT_PGI(0x1a20c),
	ALWAYS_SOFT_HINT_PGI(0x1a20d),
	ALLOW_DRAW_OBJ_HINT_PGI(0x1a20e),
	ALLOW_DRAW_WIN_HINT_PGI(0x1a20f),
	ALLOW_DRAW_FRG_HINT_PGI(0x1a210),
	ALLOW_DRAW_MEM_HINT_PGI(0x1a211),
	STRICT_DEPTHFUNC_HINT_PGI(0x1a216),
	STRICT_LIGHTING_HINT_PGI(0x1a217),
	STRICT_SCISSOR_HINT_PGI(0x1a218),
	FULL_STIPPLE_HINT_PGI(0x1a219),
	CLIP_NEAR_HINT_PGI(0x1a220),
	CLIP_FAR_HINT_PGI(0x1a221),
	WIDE_LINE_HINT_PGI(0x1a222),
	BACK_NORMALS_HINT_PGI(0x1a223),
	VERTEX_DATA_HINT_PGI(0x1a22a),
	VERTEX_CONSISTENT_HINT_PGI(0x1a22b),
	MATERIAL_SIDE_HINT_PGI(0x1a22c),
	MAX_VERTEX_HINT_PGI(0x1a22d);
	
	private final int glEnum;

	GLHintTarget(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLHintTarget> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLHintTarget fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0xc50 -> PERSPECTIVE_CORRECTION_HINT;
			case 0xc51 -> POINT_SMOOTH_HINT;
			case 0xc52 -> LINE_SMOOTH_HINT;
			case 0xc53 -> POLYGON_SMOOTH_HINT;
			case 0xc54 -> FOG_HINT;
			case 0x800e -> PACK_CMYK_HINT_EXT;
			case 0x800f -> UNPACK_CMYK_HINT_EXT;
			case 0x80eb -> PHONG_HINT_WIN;
			case 0x80f0 -> CLIP_VOLUME_CLIPPING_HINT_EXT;
			case 0x812e -> TEXTURE_MULTI_BUFFER_HINT_SGIX;
			case 0x8192 -> GENERATE_MIPMAP_HINT;
			case 0x8257 -> PROGRAM_BINARY_RETRIEVABLE_HINT;
			case 0x8316 -> CONVOLUTION_HINT_SGIX;
			case 0x8322 -> SCALEBIAS_HINT_SGIX;
			case 0x835b -> LINE_QUALITY_HINT_SGIX;
			case 0x83ee -> VERTEX_PRECLIP_SGIX;
			case 0x83ef -> VERTEX_PRECLIP_HINT_SGIX;
			case 0x84ef -> TEXTURE_COMPRESSION_HINT;
			case 0x851f -> VERTEX_ARRAY_STORAGE_HINT_APPLE;
			case 0x8534 -> MULTISAMPLE_FILTER_HINT_NV;
			case 0x85b1 -> TRANSFORM_HINT_APPLE;
			case 0x85bc -> TEXTURE_STORAGE_HINT_APPLE;
			case 0x8b8b -> FRAGMENT_SHADER_DERIVATIVE_HINT;
			case 0x8fb0 -> BINNING_CONTROL_HINT_QCOM;
			case 0x1a1f8 -> PREFER_DOUBLEBUFFER_HINT_PGI;
			case 0x1a1fd -> CONSERVE_MEMORY_HINT_PGI;
			case 0x1a1fe -> RECLAIM_MEMORY_HINT_PGI;
			case 0x1a203 -> NATIVE_GRAPHICS_BEGIN_HINT_PGI;
			case 0x1a204 -> NATIVE_GRAPHICS_END_HINT_PGI;
			case 0x1a20c -> ALWAYS_FAST_HINT_PGI;
			case 0x1a20d -> ALWAYS_SOFT_HINT_PGI;
			case 0x1a20e -> ALLOW_DRAW_OBJ_HINT_PGI;
			case 0x1a20f -> ALLOW_DRAW_WIN_HINT_PGI;
			case 0x1a210 -> ALLOW_DRAW_FRG_HINT_PGI;
			case 0x1a211 -> ALLOW_DRAW_MEM_HINT_PGI;
			case 0x1a216 -> STRICT_DEPTHFUNC_HINT_PGI;
			case 0x1a217 -> STRICT_LIGHTING_HINT_PGI;
			case 0x1a218 -> STRICT_SCISSOR_HINT_PGI;
			case 0x1a219 -> FULL_STIPPLE_HINT_PGI;
			case 0x1a220 -> CLIP_NEAR_HINT_PGI;
			case 0x1a221 -> CLIP_FAR_HINT_PGI;
			case 0x1a222 -> WIDE_LINE_HINT_PGI;
			case 0x1a223 -> BACK_NORMALS_HINT_PGI;
			case 0x1a22a -> VERTEX_DATA_HINT_PGI;
			case 0x1a22b -> VERTEX_CONSISTENT_HINT_PGI;
			case 0x1a22c -> MATERIAL_SIDE_HINT_PGI;
			case 0x1a22d -> MAX_VERTEX_HINT_PGI;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLHintTarget");
		};
	}
}