package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLPixelType implements GLEnum {
	BYTE(0x1400),
	UNSIGNED_BYTE(0x1401),
	SHORT(0x1402),
	UNSIGNED_SHORT(0x1403),
	INT(0x1404),
	UNSIGNED_INT(0x1405),
	FLOAT(0x1406),
	HALF_FLOAT(0x140b),
	BITMAP(0x1a00),
	UNSIGNED_BYTE_3_3_2(0x8032),
	UNSIGNED_SHORT_4_4_4_4(0x8033),
	UNSIGNED_SHORT_5_5_5_1(0x8034),
	UNSIGNED_INT_8_8_8_8(0x8035),
	UNSIGNED_INT_10_10_10_2(0x8036),
	UNSIGNED_BYTE_2_3_3_REV(0x8362),
	UNSIGNED_SHORT_5_6_5(0x8363),
	UNSIGNED_SHORT_5_6_5_REV(0x8364),
	UNSIGNED_SHORT_4_4_4_4_REV(0x8365),
	UNSIGNED_SHORT_1_5_5_5_REV(0x8366),
	UNSIGNED_INT_8_8_8_8_REV(0x8367),
	UNSIGNED_INT_2_10_10_10_REV(0x8368),
	UNSIGNED_INT_24_8(0x84fa),
	UNSIGNED_INT_10F_11F_11F_REV(0x8c3b),
	UNSIGNED_INT_5_9_9_9_REV(0x8c3e),
	FLOAT_32_UNSIGNED_INT_24_8_REV(0x8dad);
	
	private final int glEnum;

	GLPixelType(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLPixelType> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLPixelType fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1400 -> BYTE;
			case 0x1401 -> UNSIGNED_BYTE;
			case 0x1402 -> SHORT;
			case 0x1403 -> UNSIGNED_SHORT;
			case 0x1404 -> INT;
			case 0x1405 -> UNSIGNED_INT;
			case 0x1406 -> FLOAT;
			case 0x140b -> HALF_FLOAT;
			case 0x1a00 -> BITMAP;
			case 0x8032 -> UNSIGNED_BYTE_3_3_2;
			case 0x8033 -> UNSIGNED_SHORT_4_4_4_4;
			case 0x8034 -> UNSIGNED_SHORT_5_5_5_1;
			case 0x8035 -> UNSIGNED_INT_8_8_8_8;
			case 0x8036 -> UNSIGNED_INT_10_10_10_2;
			case 0x8362 -> UNSIGNED_BYTE_2_3_3_REV;
			case 0x8363 -> UNSIGNED_SHORT_5_6_5;
			case 0x8364 -> UNSIGNED_SHORT_5_6_5_REV;
			case 0x8365 -> UNSIGNED_SHORT_4_4_4_4_REV;
			case 0x8366 -> UNSIGNED_SHORT_1_5_5_5_REV;
			case 0x8367 -> UNSIGNED_INT_8_8_8_8_REV;
			case 0x8368 -> UNSIGNED_INT_2_10_10_10_REV;
			case 0x84fa -> UNSIGNED_INT_24_8;
			case 0x8c3b -> UNSIGNED_INT_10F_11F_11F_REV;
			case 0x8c3e -> UNSIGNED_INT_5_9_9_9_REV;
			case 0x8dad -> FLOAT_32_UNSIGNED_INT_24_8_REV;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLPixelType");
		};
	}
}