package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLReadBufferMode implements GLEnum {
	NONE(0x0),
	FRONT_LEFT(0x400),
	FRONT_RIGHT(0x401),
	BACK_LEFT(0x402),
	BACK_RIGHT(0x403),
	FRONT(0x404),
	BACK(0x405),
	LEFT(0x406),
	RIGHT(0x407),
	AUX0(0x409),
	AUX1(0x40a),
	AUX2(0x40b),
	AUX3(0x40c),
	COLOR_ATTACHMENT0(0x8ce0),
	COLOR_ATTACHMENT1(0x8ce1),
	COLOR_ATTACHMENT2(0x8ce2),
	COLOR_ATTACHMENT3(0x8ce3),
	COLOR_ATTACHMENT4(0x8ce4),
	COLOR_ATTACHMENT5(0x8ce5),
	COLOR_ATTACHMENT6(0x8ce6),
	COLOR_ATTACHMENT7(0x8ce7),
	COLOR_ATTACHMENT8(0x8ce8),
	COLOR_ATTACHMENT9(0x8ce9),
	COLOR_ATTACHMENT10(0x8cea),
	COLOR_ATTACHMENT11(0x8ceb),
	COLOR_ATTACHMENT12(0x8cec),
	COLOR_ATTACHMENT13(0x8ced),
	COLOR_ATTACHMENT14(0x8cee),
	COLOR_ATTACHMENT15(0x8cef);
	
	private final int glEnum;

	GLReadBufferMode(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLReadBufferMode> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLReadBufferMode fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x0 -> NONE;
			case 0x400 -> FRONT_LEFT;
			case 0x401 -> FRONT_RIGHT;
			case 0x402 -> BACK_LEFT;
			case 0x403 -> BACK_RIGHT;
			case 0x404 -> FRONT;
			case 0x405 -> BACK;
			case 0x406 -> LEFT;
			case 0x407 -> RIGHT;
			case 0x409 -> AUX0;
			case 0x40a -> AUX1;
			case 0x40b -> AUX2;
			case 0x40c -> AUX3;
			case 0x8ce0 -> COLOR_ATTACHMENT0;
			case 0x8ce1 -> COLOR_ATTACHMENT1;
			case 0x8ce2 -> COLOR_ATTACHMENT2;
			case 0x8ce3 -> COLOR_ATTACHMENT3;
			case 0x8ce4 -> COLOR_ATTACHMENT4;
			case 0x8ce5 -> COLOR_ATTACHMENT5;
			case 0x8ce6 -> COLOR_ATTACHMENT6;
			case 0x8ce7 -> COLOR_ATTACHMENT7;
			case 0x8ce8 -> COLOR_ATTACHMENT8;
			case 0x8ce9 -> COLOR_ATTACHMENT9;
			case 0x8cea -> COLOR_ATTACHMENT10;
			case 0x8ceb -> COLOR_ATTACHMENT11;
			case 0x8cec -> COLOR_ATTACHMENT12;
			case 0x8ced -> COLOR_ATTACHMENT13;
			case 0x8cee -> COLOR_ATTACHMENT14;
			case 0x8cef -> COLOR_ATTACHMENT15;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLReadBufferMode");
		};
	}
}