package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLTextureEnvParameter implements GLEnum {
	ALPHA_SCALE(0xd1c),
	TEXTURE_ENV_MODE(0x2200),
	TEXTURE_ENV_COLOR(0x2201),
	TEXTURE_LOD_BIAS(0x8501),
	COMBINE(0x8570),
	COMBINE_RGB(0x8571),
	COMBINE_ALPHA(0x8572),
	RGB_SCALE(0x8573),
	ADD_SIGNED(0x8574),
	INTERPOLATE(0x8575),
	CONSTANT(0x8576),
	PRIMARY_COLOR(0x8577),
	PREVIOUS(0x8578),
	SOURCE0_RGB(0x8580),
	SOURCE1_RGB(0x8581),
	SOURCE2_RGB(0x8582),
	SOURCE3_RGB_NV(0x8583),
	SOURCE0_ALPHA(0x8588),
	SOURCE1_ALPHA(0x8589),
	SOURCE2_ALPHA(0x858a),
	SOURCE3_ALPHA_NV(0x858b),
	OPERAND0_RGB(0x8590),
	OPERAND1_RGB(0x8591),
	OPERAND2_RGB(0x8592),
	OPERAND3_RGB_NV(0x8593),
	OPERAND0_ALPHA(0x8598),
	OPERAND1_ALPHA(0x8599),
	OPERAND2_ALPHA(0x859a),
	OPERAND3_ALPHA_NV(0x859b),
	COORD_REPLACE(0x8862);
	
	private final int glEnum;

	GLTextureEnvParameter(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLTextureEnvParameter> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLTextureEnvParameter fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0xd1c -> ALPHA_SCALE;
			case 0x2200 -> TEXTURE_ENV_MODE;
			case 0x2201 -> TEXTURE_ENV_COLOR;
			case 0x8501 -> TEXTURE_LOD_BIAS;
			case 0x8570 -> COMBINE;
			case 0x8571 -> COMBINE_RGB;
			case 0x8572 -> COMBINE_ALPHA;
			case 0x8573 -> RGB_SCALE;
			case 0x8574 -> ADD_SIGNED;
			case 0x8575 -> INTERPOLATE;
			case 0x8576 -> CONSTANT;
			case 0x8577 -> PRIMARY_COLOR;
			case 0x8578 -> PREVIOUS;
			case 0x8580 -> SOURCE0_RGB;
			case 0x8581 -> SOURCE1_RGB;
			case 0x8582 -> SOURCE2_RGB;
			case 0x8583 -> SOURCE3_RGB_NV;
			case 0x8588 -> SOURCE0_ALPHA;
			case 0x8589 -> SOURCE1_ALPHA;
			case 0x858a -> SOURCE2_ALPHA;
			case 0x858b -> SOURCE3_ALPHA_NV;
			case 0x8590 -> OPERAND0_RGB;
			case 0x8591 -> OPERAND1_RGB;
			case 0x8592 -> OPERAND2_RGB;
			case 0x8593 -> OPERAND3_RGB_NV;
			case 0x8598 -> OPERAND0_ALPHA;
			case 0x8599 -> OPERAND1_ALPHA;
			case 0x859a -> OPERAND2_ALPHA;
			case 0x859b -> OPERAND3_ALPHA_NV;
			case 0x8862 -> COORD_REPLACE;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLTextureEnvParameter");
		};
	}
}