package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLUniformType implements GLEnum {
	INT(0x1404),
	UNSIGNED_INT(0x1405),
	FLOAT(0x1406),
	DOUBLE(0x140a),
	FLOAT_VEC2(0x8b50),
	FLOAT_VEC3(0x8b51),
	FLOAT_VEC4(0x8b52),
	INT_VEC2(0x8b53),
	INT_VEC3(0x8b54),
	INT_VEC4(0x8b55),
	BOOL(0x8b56),
	BOOL_VEC2(0x8b57),
	BOOL_VEC3(0x8b58),
	BOOL_VEC4(0x8b59),
	FLOAT_MAT2(0x8b5a),
	FLOAT_MAT3(0x8b5b),
	FLOAT_MAT4(0x8b5c),
	SAMPLER_1D(0x8b5d),
	SAMPLER_2D(0x8b5e),
	SAMPLER_3D(0x8b5f),
	SAMPLER_CUBE(0x8b60),
	SAMPLER_1D_SHADOW(0x8b61),
	SAMPLER_2D_SHADOW(0x8b62),
	SAMPLER_2D_RECT(0x8b63),
	SAMPLER_2D_RECT_SHADOW(0x8b64),
	FLOAT_MAT2x3(0x8b65),
	FLOAT_MAT2x4(0x8b66),
	FLOAT_MAT3x2(0x8b67),
	FLOAT_MAT3x4(0x8b68),
	FLOAT_MAT4x2(0x8b69),
	FLOAT_MAT4x3(0x8b6a),
	SAMPLER_1D_ARRAY(0x8dc0),
	SAMPLER_2D_ARRAY(0x8dc1),
	SAMPLER_BUFFER(0x8dc2),
	SAMPLER_1D_ARRAY_SHADOW(0x8dc3),
	SAMPLER_2D_ARRAY_SHADOW(0x8dc4),
	SAMPLER_CUBE_SHADOW(0x8dc5),
	UNSIGNED_INT_VEC2(0x8dc6),
	UNSIGNED_INT_VEC3(0x8dc7),
	UNSIGNED_INT_VEC4(0x8dc8),
	INT_SAMPLER_1D(0x8dc9),
	INT_SAMPLER_2D(0x8dca),
	INT_SAMPLER_3D(0x8dcb),
	INT_SAMPLER_CUBE(0x8dcc),
	INT_SAMPLER_2D_RECT(0x8dcd),
	INT_SAMPLER_1D_ARRAY(0x8dce),
	INT_SAMPLER_2D_ARRAY(0x8dcf),
	INT_SAMPLER_BUFFER(0x8dd0),
	UNSIGNED_INT_SAMPLER_1D(0x8dd1),
	UNSIGNED_INT_SAMPLER_2D(0x8dd2),
	UNSIGNED_INT_SAMPLER_3D(0x8dd3),
	UNSIGNED_INT_SAMPLER_CUBE(0x8dd4),
	UNSIGNED_INT_SAMPLER_2D_RECT(0x8dd5),
	UNSIGNED_INT_SAMPLER_1D_ARRAY(0x8dd6),
	UNSIGNED_INT_SAMPLER_2D_ARRAY(0x8dd7),
	UNSIGNED_INT_SAMPLER_BUFFER(0x8dd8),
	DOUBLE_MAT2(0x8f46),
	DOUBLE_MAT3(0x8f47),
	DOUBLE_MAT4(0x8f48),
	DOUBLE_MAT2x3(0x8f49),
	DOUBLE_MAT2x4(0x8f4a),
	DOUBLE_MAT3x2(0x8f4b),
	DOUBLE_MAT3x4(0x8f4c),
	DOUBLE_MAT4x2(0x8f4d),
	DOUBLE_MAT4x3(0x8f4e),
	DOUBLE_VEC2(0x8ffc),
	DOUBLE_VEC3(0x8ffd),
	DOUBLE_VEC4(0x8ffe),
	SAMPLER_CUBE_MAP_ARRAY(0x900c),
	SAMPLER_CUBE_MAP_ARRAY_SHADOW(0x900d),
	INT_SAMPLER_CUBE_MAP_ARRAY(0x900e),
	UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY(0x900f),
	SAMPLER_2D_MULTISAMPLE(0x9108),
	INT_SAMPLER_2D_MULTISAMPLE(0x9109),
	UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE(0x910a),
	SAMPLER_2D_MULTISAMPLE_ARRAY(0x910b),
	INT_SAMPLER_2D_MULTISAMPLE_ARRAY(0x910c),
	UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY(0x910d);
	
	private final int glEnum;

	GLUniformType(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLUniformType> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLUniformType fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1404 -> INT;
			case 0x1405 -> UNSIGNED_INT;
			case 0x1406 -> FLOAT;
			case 0x140a -> DOUBLE;
			case 0x8b50 -> FLOAT_VEC2;
			case 0x8b51 -> FLOAT_VEC3;
			case 0x8b52 -> FLOAT_VEC4;
			case 0x8b53 -> INT_VEC2;
			case 0x8b54 -> INT_VEC3;
			case 0x8b55 -> INT_VEC4;
			case 0x8b56 -> BOOL;
			case 0x8b57 -> BOOL_VEC2;
			case 0x8b58 -> BOOL_VEC3;
			case 0x8b59 -> BOOL_VEC4;
			case 0x8b5a -> FLOAT_MAT2;
			case 0x8b5b -> FLOAT_MAT3;
			case 0x8b5c -> FLOAT_MAT4;
			case 0x8b5d -> SAMPLER_1D;
			case 0x8b5e -> SAMPLER_2D;
			case 0x8b5f -> SAMPLER_3D;
			case 0x8b60 -> SAMPLER_CUBE;
			case 0x8b61 -> SAMPLER_1D_SHADOW;
			case 0x8b62 -> SAMPLER_2D_SHADOW;
			case 0x8b63 -> SAMPLER_2D_RECT;
			case 0x8b64 -> SAMPLER_2D_RECT_SHADOW;
			case 0x8b65 -> FLOAT_MAT2x3;
			case 0x8b66 -> FLOAT_MAT2x4;
			case 0x8b67 -> FLOAT_MAT3x2;
			case 0x8b68 -> FLOAT_MAT3x4;
			case 0x8b69 -> FLOAT_MAT4x2;
			case 0x8b6a -> FLOAT_MAT4x3;
			case 0x8dc0 -> SAMPLER_1D_ARRAY;
			case 0x8dc1 -> SAMPLER_2D_ARRAY;
			case 0x8dc2 -> SAMPLER_BUFFER;
			case 0x8dc3 -> SAMPLER_1D_ARRAY_SHADOW;
			case 0x8dc4 -> SAMPLER_2D_ARRAY_SHADOW;
			case 0x8dc5 -> SAMPLER_CUBE_SHADOW;
			case 0x8dc6 -> UNSIGNED_INT_VEC2;
			case 0x8dc7 -> UNSIGNED_INT_VEC3;
			case 0x8dc8 -> UNSIGNED_INT_VEC4;
			case 0x8dc9 -> INT_SAMPLER_1D;
			case 0x8dca -> INT_SAMPLER_2D;
			case 0x8dcb -> INT_SAMPLER_3D;
			case 0x8dcc -> INT_SAMPLER_CUBE;
			case 0x8dcd -> INT_SAMPLER_2D_RECT;
			case 0x8dce -> INT_SAMPLER_1D_ARRAY;
			case 0x8dcf -> INT_SAMPLER_2D_ARRAY;
			case 0x8dd0 -> INT_SAMPLER_BUFFER;
			case 0x8dd1 -> UNSIGNED_INT_SAMPLER_1D;
			case 0x8dd2 -> UNSIGNED_INT_SAMPLER_2D;
			case 0x8dd3 -> UNSIGNED_INT_SAMPLER_3D;
			case 0x8dd4 -> UNSIGNED_INT_SAMPLER_CUBE;
			case 0x8dd5 -> UNSIGNED_INT_SAMPLER_2D_RECT;
			case 0x8dd6 -> UNSIGNED_INT_SAMPLER_1D_ARRAY;
			case 0x8dd7 -> UNSIGNED_INT_SAMPLER_2D_ARRAY;
			case 0x8dd8 -> UNSIGNED_INT_SAMPLER_BUFFER;
			case 0x8f46 -> DOUBLE_MAT2;
			case 0x8f47 -> DOUBLE_MAT3;
			case 0x8f48 -> DOUBLE_MAT4;
			case 0x8f49 -> DOUBLE_MAT2x3;
			case 0x8f4a -> DOUBLE_MAT2x4;
			case 0x8f4b -> DOUBLE_MAT3x2;
			case 0x8f4c -> DOUBLE_MAT3x4;
			case 0x8f4d -> DOUBLE_MAT4x2;
			case 0x8f4e -> DOUBLE_MAT4x3;
			case 0x8ffc -> DOUBLE_VEC2;
			case 0x8ffd -> DOUBLE_VEC3;
			case 0x8ffe -> DOUBLE_VEC4;
			case 0x900c -> SAMPLER_CUBE_MAP_ARRAY;
			case 0x900d -> SAMPLER_CUBE_MAP_ARRAY_SHADOW;
			case 0x900e -> INT_SAMPLER_CUBE_MAP_ARRAY;
			case 0x900f -> UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY;
			case 0x9108 -> SAMPLER_2D_MULTISAMPLE;
			case 0x9109 -> INT_SAMPLER_2D_MULTISAMPLE;
			case 0x910a -> UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE;
			case 0x910b -> SAMPLER_2D_MULTISAMPLE_ARRAY;
			case 0x910c -> INT_SAMPLER_2D_MULTISAMPLE_ARRAY;
			case 0x910d -> UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLUniformType");
		};
	}
}