package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLUseProgramStageMask implements GLEnum {
	VERTEX_SHADER_BIT(0x1),
	FRAGMENT_SHADER_BIT(0x2),
	GEOMETRY_SHADER_BIT(0x4),
	TESS_CONTROL_SHADER_BIT(0x8),
	TESS_EVALUATION_SHADER_BIT(0x10),
	COMPUTE_SHADER_BIT(0x20),
	MESH_SHADER_BIT_NV(0x40),
	TASK_SHADER_BIT_NV(0x80),
	ALL_SHADER_BITS(0xffffffff);
	
	private final int glEnum;

	GLUseProgramStageMask(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLUseProgramStageMask> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLUseProgramStageMask fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1 -> VERTEX_SHADER_BIT;
			case 0x2 -> FRAGMENT_SHADER_BIT;
			case 0x4 -> GEOMETRY_SHADER_BIT;
			case 0x8 -> TESS_CONTROL_SHADER_BIT;
			case 0x10 -> TESS_EVALUATION_SHADER_BIT;
			case 0x20 -> COMPUTE_SHADER_BIT;
			case 0x40 -> MESH_SHADER_BIT_NV;
			case 0x80 -> TASK_SHADER_BIT_NV;
			case 0xffffffff -> ALL_SHADER_BITS;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLUseProgramStageMask");
		};
	}
}